@@ -108,21 +108,22 @@ str.emmGrid <- function(object, ...) {
 
 
 #' @rdname emmGrid-methods
-#' @param export Logical value. If \code{FALSE}, the object is printed. 
+#' @param export Logical value. If \code{FALSE}, the object is printed.
 #'   If \code{TRUE}, a list is invisibly returned, which contains character
-#'   elements named \code{summary} and \code{annotations} that may be saved 
-#'   or displayed as the user sees fit. \code{summary} is a character matrix 
+#'   elements named \code{summary} and \code{annotations} that may be saved
+#'   or displayed as the user sees fit. \code{summary} is a character matrix
 #'   (or list of such matrices, if a \code{by} variable is in effect).
-#'   \code{annotations} is a character vector of the annotations that would 
+#'   \code{annotations} is a character vector of the annotations that would
 #'   have been printed below the summary or summaries.
 #' @method print emmGrid
 #' @param x An \code{emmGrid} object
 #' @export
 print.emmGrid = function(x, ..., export = FALSE) {
-    if(export) 
-        print(summary.emmGrid(x, ...), export = export)
-    else
-        show(x)
+    print(summary.emmGrid(x, ...), export = export)
+    # if(export)
+    #     print(summary.emmGrid(x, ...), export = export)
+    # else
+    #     show(x)
 }
 
 ### Former print method which I changed to work around a bug in pkgdown
@@ -172,6 +173,18 @@ vcov.emmGrid = function(object, ..., sep = get_emm_option("sep")) {
 }
 
 
+#' @rdname emmGrid-methods
+#'
+#' @returns The \code{linfct} function and method returns the \code{linfct} slot of \code{object}.
+#' @export
+linfct = function(object, ...)
+    UseMethod("linfct")
+
+#' @rdname emmGrid-methods
+#' @export
+linfct.default = function(object, ...) {
+    attr(object, "linfct")
+}
 
 
 # Method to alter contents of misc slot
@@ -521,7 +534,7 @@ update.emmGrid = function(object, ..., silent = FALSE) {
 #' \item{\code{summary}}{A named \code{list} of defaults used by the methods
 #'   \code{\link{summary.emmGrid}}, \code{\link{predict.emmGrid}}, \code{\link{test.emmGrid}},
 #'   \code{\link{confint.emmGrid}}, and \code{\link{emmip}}. The only option that can
-#'   affect the latter four is \code{"predict.method"}.}
+#'   affect the latter four is \code{"predict.type"}.}
 #' \item{\code{allow.na.levs}}{A logical value that if \code{TRUE} (the default), allows
 #'   \code{NA} to be among the levels of a factor. Older versions of \pkg{emmeans} did not
 #'   allow this. So if problems come up (say in a messy dataset that includes incomplete cases), 
@@ -541,6 +554,9 @@ update.emmGrid = function(object, ..., silent = FALSE) {
 #' \item{\code{graphics.engine}}{A character value matching 
 #'   \code{c("ggplot", "lattice")}, setting the default engine to use in
 #'   \code{\link{emmip}} and \code{\link{plot.emmGrid}}.  Defaults to \code{"ggplot"}.}
+#' \item{\code{gg.theme}}{An integer. Set it to 1 or 2 if you want the appearance
+#'   of plots used in \pkg{emmeans} version 1.x.x or 2.x.x. Or set it to the actual theme
+#'   that you want to use, e.g., \code{emm_options(gg.theme = ggplot2::theme_dark())}}
 #' \item{\code{msg.interaction}}{A logical value controlling whether or not
 #'   a message is displayed when \code{emmeans} averages over a factor involved
 #'   in an interaction. It is probably not appropriate to do this, unless
@@ -569,6 +585,10 @@ update.emmGrid = function(object, ..., silent = FALSE) {
 #'   displayed is just enough to reasonably distinguish estimates from the ends
 #'   of their confidence intervals; but always at least 3 digits. If
 #'   \code{FALSE}, the system value \code{getOption("digits")} is used.}
+#' \item{\code{pval.digits}}{An integer indicating the precision with which
+#'   p-values are printed. Integers between \code{2} and \code{6} are acceptable; the default is 
+#' \code{4}. Floating point numbers are truncated. If a number outside the accepted range is
+#' used, the nearest acceptable integer will be used instead. Non-numeric arguments are ignored.}
 #' \item{\code{back.bias.adj}}{A logical value controlling whether we 
 #'   try to adjust bias when back-transforming. If \code{FALSE}, we use naive
 #'   back transformation. If \code{TRUE} \emph{and \code{sigma} is available and valid}, a
@@ -594,6 +614,8 @@ update.emmGrid = function(object, ..., silent = FALSE) {
 #' options affect how degrees of freedom are computed for \code{lmerMod} objects
 #' produced by the \pkg{lme4} package). See that section of the "models" vignette
 #' for details.}
+#' \item{\code{post.ci.method}}{method for estimating confidence intervals for Bayesian models. 
+#' Options are \code{"HPD"} (default) or \code{"quantile"}.} 
 #' } %%%%%% end \describe
 #'
 #' @param ... Option names and values (see Details)
@@ -621,6 +643,13 @@ update.emmGrid = function(object, ..., silent = FALSE) {
 #' \code{options(emmeans = get_emm_option("saved.opts"))}. To completely erase
 #' all options, use \code{options(emmeans = NULL)}
 #' 
+#' @section Startup options:
+#' \pkg{emmeans}'s options are kept in the system options in a list names \code{emmeans}.
+#' Thus, if the user wants certain options set on startup, this can be arranged by
+#' setting the \code{emmeans} option in one's startup script. For example,
+#' the user's \code{.Rprofile} file could contain:
+#' \preformatted{    options(emmeans = list(summary = list(predict.type = "response"), 
+#'                            lmer.df = "satterthwaite"))}
 #' 
 #' 
 #' @seealso \code{\link{update.emmGrid}}
@@ -700,10 +729,11 @@ get_emm_option = function(x, default = emm_defaults[[x]]) {
 #' @export
 #' @examples
 #' 
-#' # Illustration of how 'opt.digits' affects the results of print()
-#' # Note that the returned value is printed with the default setting (opt.digits = TRUE)
-#' pigs.lm <- lm(inverse(conc) ~ source, data = pigs)
-#' with_emm_options(opt.digits = FALSE, print(emmeans(pigs.lm, "source")))
+#' # Temporarily alter the formatting of labels in an interaction contrast:
+#' pigs.lm <- lm(inverse(conc) ~ source * factor(percent), data = pigs)
+#' with_emm_options(parens = c("\\-", "<", ">"), sep = ": ",
+#'     contrast(contrast(ref_grid(pigs.lm), "consec", by = "percent"),
+#'         "consec", by = NULL))
 #' 
 with_emm_options = function(..., expr) {
     cl = match.call()
@@ -714,7 +744,7 @@ with_emm_options = function(..., expr) {
     cl[[1]] = as.name("emm_options")
     oldopts = getOption("emmeans")
     eval(cl, parent.frame())
-    result = try(eval(expr, parent.frame()))
+    result = try(eval(expr, parent.frame()), silent = TRUE)
     options(emmeans = oldopts)
     result
 }
@@ -734,20 +764,24 @@ emm_defaults = list (
     sep = " ",                # separator for combining factor levels
     parens = c("-|\\+|\\/|\\*", "(", ")"), # patterns for what/how to parenthesize in contrast
     graphics.engine = "ggplot",  # default for emmip and plot.emmGrid
+    gg.theme = 2,             # Use the ggplot theme used in this emmeans version 
 ###    msg.data.call = TRUE,     # message when there's a call in data or subset
     msg.interaction = TRUE,   # message about averaging w/ interactions
     msg.nesting = TRUE,       # message when nesting is detected
     estble.tol = 1e-8,        # tolerance for estimability checks
     simplify.names = TRUE,    # simplify names like data$x to just "x"
     back.bias.adj = FALSE,    # Try to bias-adjust back-transformations?
-    opt.digits = TRUE,        # optimize displayed digits?
+    opt.digits = TRUE,        # optimize displayed digits for estimates and CI's?
+    pval.digits = 4L,          # how many digits to list for p-values
     enable.submodel = TRUE,   # enable saving extra info for submodel
     rg.limit = 10000,         # limit on number of rows in a reference grid
     lmer.df = "kenward-roger",  # Use Kenward-Roger for df
     disable.pbkrtest = FALSE, # whether to bypass pbkrtest routines for lmerMod
     pbkrtest.limit = 3000,    # limit on N for enabling K-R
     disable.lmerTest = FALSE, # whether to bypass lmerTest routines for lmerMod
-    lmerTest.limit = 3000     # limit on N for enabling Satterthwaite
+    lmerTest.limit = 3000,     # limit on N for enabling Satterthwaite
+    post.ci.method = "HPD"    #  set CI method for Bayesian methods
+
 )
 
 
