@@ -40,11 +40,24 @@
 #' \code{ref_grid}.
 #'
 #' The reference grid consists of combinations of independent variables over
-#' which predictions are made. Estimated marginal means are defined as these
+#' which predictions are made. 
+#' Estimated marginal means are defined as these
 #' predictions, or marginal averages thereof. The grid is determined by first
 #' reconstructing the data used in fitting the model (see
 #' \code{\link{recover_data}}), or by using the \code{data.frame} provided in
-#' \code{data}. The default reference grid is determined by the observed levels
+#' \code{data}. 
+#' 
+#' By \dQuote{independent variables,} we mean (in most cases) the results of
+#' \code{all.vars()} applied to the fixed-effects part of the right-hand side of
+#' the model formula. Any random effects are excluded. Thus, if the model
+#' formula in an \code{lme4::lmer} call is \code{yield ~ fert +
+#' seed*density + log(rain) + (1|block/plot)}, the independent variables are
+#' \code{fert}, \code{seed}, \code{density}, and \code{rain} (not
+#' \code{log(rain)}). In multivariate models, the dimension of the multivariate
+#' response is also considered an independent variable.
+#' 
+#' 
+#' The default reference grid is determined by the observed levels
 #' of any factors, the ordered unique values of character-valued predictors, and
 #' the results of \code{cov.reduce} for numeric predictors. These may be
 #' overridden using \code{at}. See also the section below on
@@ -107,16 +120,13 @@
 #'   \code{sigma(object)}, if available, and \code{NULL} otherwise.
 #'   Note: This applies only when the family is \code{"gaussian"}; for other families,
 #'   \code{sigma} is set to \code{NA} and cannot be overridden.
-#' @param counterfactuals,wt.counter,avg.counter \code{counterfactuals} specifies character
+#' @param counterfactuals \code{counterfactuals} specifies character
 #'   names of counterfactual factors. If this is non-missing, a reference grid
-#'   is created consisting of combinations of counterfactual levels and a constructed
-#'   factor \code{.obs.no.} having a level for each observation in the dataset.
-#'   By default, this grid is re-gridded with the response transformation
-#'   and averaged over \code{.obs.no.} (by default, with equal weights, but
-#'   a vector of weights may be specified in \code{wt.counter}; it must be
-#'   of length equal to the number of observations in the dataset).
-#'   If \code{avg.counter} is set to \code{FALSE}, this averaging is disabled.
-#'   See the section below on counterfactuals.
+#'   is created consisting of combinations of counterfactual levels 
+#'   and the actual levels of those same factors.
+#'   This grid is always converted to the response transformation scale
+#'   and averaged over the actual factor levels. See the section below 
+#'   on counterfactuals. 
 #' @param nuisance,non.nuisance,wt.nuis If \code{nuisance} is a vector of predictor names,
 #'   those predictors are omitted from the reference grid. Instead, the result 
 #'   will be as if we had averaged over the levels of those factors, with either 
@@ -125,7 +135,10 @@
 #'   \code{nuisance} must not interact with other factors, not even other
 #'   nuisance factors. Specifying nuisance factors can save considerable
 #'   storage and computation time, and help avoid exceeding the maximum
-#'   reference-grid size (\code{get_emm_option("rg.limit")}).
+#'   reference-grid size (\code{get_emm_option("rg.limit")}). 
+#'   (\emph{Note:} For certain models where the \code{emm_basis} method returns a
+#'   re-gridded parameterization, nuisance factors cannot be used, and an error
+#'   is thrown.)
 #' @param rg.limit Integer limit on the number of reference-grid rows to allow
 #'   (checked before any multivariate responses are included).
 #' @param ... Optional arguments passed to \code{\link{summary.emmGrid}},
@@ -315,23 +328,29 @@
 #'
 #' @section Counterfactuals:
 #'   If \code{counterfactuals} is specified, the rows of the entire dataset
-#'   become a factor in the reference grid, and the other reference levels are
+#'   become part of the reference grid, and the other reference levels are
 #'   confined to those named in \code{counterfactuals}. In this type of analysis
-#'   (called G-computation), we substitute each combination of counterfactual
+#'   (called G-computation), we substitute (or impute) each combination of counterfactual
 #'   levels into the entire dataset. Thus, predictions from this grid are those
 #'   of each observation under each of the counterfactual levels. For this to
 #'   make sense, we require an assumption of exchangeability of these levels.
 #'   
-#'   By default, this grid is converted to the response scale (unless otherwise
-#'   specified in \code{regrid} and averaged over the observations in the dataset.
-#'   Averaging can be disabled by setting \code{avg.counter = FALSE}, but
-#'   be warned that the resulting reference grid is potentially huge -- the
-#'   number of observations in the dataset times the number of counterfactual 
-#'   combinations, times the number of multivariate levels.
+#'   This grid is always converted to the response scale, as G-computation on
+#'   the linear-predictor scale produces the same results as ordinary weighted EMMs.
+#'   If we have counterfactual factors \code{A, B}, the reference grid also includes
+#'   factors \code{actual_A, actual_B} which are used to track which observations
+#'   originally had the \code{A, B} levels before they were changed by the 
+#'   counterfactuals code. We average the response-scale predictions for each
+#'   combination of actual levels and imputed levels (and multivariate levels, 
+#'   if any). See additional discussion of how \code{\link{emmeans}} handles
+#'   counterfactuals under that documentation.
 #'   
-#'   The counterfactuals code is still fairly rudimentary and we can't guarantee
-#'   it will always work, such as in cases of nested models. Sometimes, an error
-#'   can be averted by specifying \code{avg.counter = FALSE}.
+#'   Currently, counterfactuals are not supported when the reference grid
+#'   requires post-processing (e.g., ordinal models with \code{mode = "prob"}).
+#'   Cases where we have nested factor levels can be complicated if mixed-in with counterfactuals, 
+#'   and we make no guarantees. 
+#'   Note that past implementations included arguments \code{wt.counter} and \code{avg.counter},
+#'   which are now deprecated and are just ignored if specified.
 #' 
 #' @section Optional side effect: If the \code{save.ref_grid} option is set to
 #'   \code{TRUE} (see \code{\link{emm_options}}),
@@ -437,10 +456,16 @@ ref_grid <- function(object, at, cov.reduce = mean, cov.keep = get_emm_option("c
                      mult.names, mult.levs, 
                      options = get_emm_option("ref_grid"), data, df, type, 
                      regrid, nesting, offset, sigma, 
-                     counterfactuals, wt.counter, avg.counter = TRUE,
+                     counterfactuals, ## wt.counter, avg.counter = TRUE,
                      nuisance = character(0), non.nuisance, wt.nuis = "equal",
                       rg.limit = get_emm_option("rg.limit"), ...) 
 {
+    if(!missing(counterfactuals)) { # route this to a different routine
+        cl = match.call()
+        cl[[1]] = as.name(".cf.refgrid") # internal function for counterfactuals
+        return(eval(cl))
+    }
+    
     # hack to ignore 'tran' in dots arguments and interpret 'transform' as `regrid` :
     .foo = function(t,tr,tra,tran, transform = NULL, ...) transform
     .bar = .foo(...)
@@ -548,8 +573,8 @@ ref_grid <- function(object, at, cov.reduce = mean, cov.keep = get_emm_option("c
         
         # Save the original levels of factors, no matter what
         if (is.factor(x) && !(nm %in% coerced$covariates))
-            xlev[[nm]] = levels(factor(x))
-            # (applying factor drops any unused levels)
+            xlev[[nm]] = levels(.chk.fac(x))
+            # (drops any used levels unless allow.na.levs option is TRUE)
         else if (is.character(x)) # just like a factor
             xlev[[nm]] = sort(unique(x))
     
@@ -559,7 +584,7 @@ ref_grid <- function(object, at, cov.reduce = mean, cov.keep = get_emm_option("c
             ref.levels[[nm]] = at[[nm]]
         # factors not in 'at'
         else if (is.factor(x) && !(nm %in% coerced$covariates))
-            ref.levels[[nm]] = levels(factor(x))
+            ref.levels[[nm]] = levels(.chk.fac(x))
         else if (is.character(x) || is.logical(x))
             ref.levels[[nm]] = chrlev[[nm]] = sort.unique(x)
         # matrices
@@ -595,17 +620,8 @@ ref_grid <- function(object, at, cov.reduce = mean, cov.keep = get_emm_option("c
 
     # Now create the reference grid
     if(no.nuis <- (length(nuisance) == 0)) {
-        if (!missing(counterfactuals)) {
-            cfac = intersect(counterfactuals, names(ref.levels))
-            ref.levels = ref.levels[cfac]
-            ref.levels$.obs.no. = seq_len(nrow(data))
-            .check.grid(ref.levels, rg.limit)
-            grid = .setup.cf(ref.levels, data)
-        }
-        else {
-            .check.grid(ref.levels, rg.limit)
-            grid = do.call(expand.grid, ref.levels)
-        }
+        .check.grid(ref.levels, rg.limit)
+        grid = do.call(expand.grid, ref.levels)
     }
     else {
         nuis.info = .setup.nuis(nuisance, ref.levels, trms, rg.limit)
@@ -668,20 +684,13 @@ ref_grid <- function(object, at, cov.reduce = mean, cov.keep = get_emm_option("c
     
     # we've added args `misc` and `options` so emm_basis methods can access and use these if they want
     basis = emm_basis(object, trms, xl, grid, misc = attr(data, "misc"), options = options, ...)
+
     environment(basis$dffun) = baseenv()   # releases unnecessary storage
     if(length(basis$bhat) != ncol(basis$X))
         stop("Something went wrong:\n",
              " Non-conformable elements in reference grid.",
              call. = TRUE)
     
-    collapse = NULL
-    if (!missing(counterfactuals)) {
-        grid = do.call(expand.grid, ref.levels)
-        if (missing(regrid)) 
-            regrid = "response"
-        if (avg.counter) collapse = ".obs.no."
-    }
-    
     if(!no.nuis) {
         basis = .basis.nuis(basis, nuis.info, wt.nuis, ref.levels, data, grid, ref.levels)
         grid = basis$grid
@@ -695,9 +704,10 @@ ref_grid <- function(object, at, cov.reduce = mean, cov.keep = get_emm_option("c
     # next stmt assumes that model formula is 1st argument (2nd element) in call.
     # if not, we probably get an error or something that isn't a formula
     # and it is silently ignored
-    frm = try(formula(eval(attr(data, "call")[[2]])), silent = TRUE)
+    frm = try(formula(eval(attr(data, "call")[[2]], environment(trms))), silent = TRUE)
     if (inherits(frm, "formula")) { # response may be transformed
-        lhs = frm[-3]
+        lhs = if(length(frm) == 2) NULL
+              else frm[-3]
         tran = setdiff(.all.vars(lhs, functions = TRUE), c(.all.vars(lhs), "~", "cbind", "+", "-", "*", "/", "^", "%%", "%/%"))
         if(length(tran) > 0) {
             # we are now supporting scale() as well as some functions from datawizard
@@ -798,21 +808,20 @@ ref_grid <- function(object, at, cov.reduce = mean, cov.keep = get_emm_option("c
         incl.flags = rep(TRUE, nrow(grid))
         for (nm in problems) {
             if (is.numeric(ref.levels[[nm]])) {
-                at[[nm]] = round(at[[nm]], 3)
-                ref.levels[[nm]] = round(ref.levels[[nm]], 3)
+                dig = 3 - log10(max(abs(ref.levels[[nm]])))
+                at[[nm]] = round(at[[nm]], digits = dig)
+                ref.levels[[nm]] = round(ref.levels[[nm]], digits = dig)
+                grid[[nm]] = round(grid[[nm]], digits = dig)
             }
             # get only "legal" levels
-            at[[nm]] = at[[nm]][at[[nm]] %in% ref.levels[[nm]]]
-            # Now which of those are left out?
-            excl = setdiff(ref.levels[[nm]], at[[nm]])
-            for (x in excl)
-                incl.flags[grid[[nm]] == x] = FALSE
-            ref.levels[[nm]] = at[[nm]]
+            at[[nm]] = ref.levels[[nm]] = at[[nm]][at[[nm]] %in% ref.levels[[nm]]]
+            rows = numeric(0)
+            for(x in at[[nm]])
+                rows = c(rows, which(grid[[nm]] == x))
+            grid = grid[rows, , drop = FALSE]
+            grid[[nm]] = factor(grid[[nm]], levels = at[[nm]])
+            basis$X = basis$X[rows, , drop = FALSE]
         }
-        if (!any(incl.flags))
-            stop("Reference grid is empty due to mismatched levels in 'at'")
-        grid = grid[incl.flags, , drop=FALSE]
-        basis$X = basis$X[incl.flags, , drop=FALSE]
     }
 
     # Any offsets??? (misc$offset.mult might specify removing or reversing the offset)
@@ -821,20 +830,22 @@ ref_grid <- function(object, at, cov.reduce = mean, cov.keep = get_emm_option("c
     if (!missing(offset)) {  # For safety, we always treat it as scalar
         if (offset[1] != 0)
             oval = offset[1]
+        if(".static.offset." %in% names(grid))
+            grid$.static.offset. = ref.levels$.static.offset. = oval
     }
     else {
         if(".static.offset." %in% names(grid)) { # static offset is available
             oval = om * grid[[".static.offset."]]
-            # grid$.static.offset. = NULL
         }
-        #else 
+        #else implied
         if(!is.null(attr(trms,"offset"))) {
             if (any(om != 0))
                 oval = om * (oval + .get.offset(trms, grid))
         }
-        if(any(oval != 0))
-            grid[[".offset."]] = oval
     }
+    if(any(oval != 0))
+        grid[[".offset."]] = oval
+    
     
     ### --- Determine weights for each grid point
     if (!hasName(data, "(weights)"))
@@ -899,7 +910,7 @@ ref_grid <- function(object, at, cov.reduce = mean, cov.keep = get_emm_option("c
         post.beta = matrix(NA)
     
     predictors = intersect(attr(data, "predictors"), names(grid))
-    if(!missing(counterfactuals)) predictors = c(predictors, ".obs.no.")
+    # if(!missing(counterfactuals)) predictors = c(predictors, ".obs.no.")
     
     simp.tbl = environment(trms)$.simplify.names.
     if (! is.null(simp.tbl)) {
@@ -957,11 +968,8 @@ ref_grid <- function(object, at, cov.reduce = mean, cov.keep = get_emm_option("c
         result = hook(result, ...)
     }
     if(!missing(regrid)) {
-        if(missing(wt.counter)) wt.counter = 1
-        result = regrid(result, transform = regrid, sigma = sigma, 
-                        .collapse = collapse, wt.counter = wt.counter, ...)
-        if(!is.null(collapse))
-            result@misc$avgd.over = collapse
+        # if(missing(wt.counter)) wt.counter = 1
+        result = regrid(result, transform = regrid, sigma = sigma, ...)
     }
     
     .save.ref_grid(result)
@@ -1047,13 +1055,20 @@ ref_grid <- function(object, at, cov.reduce = mean, cov.keep = get_emm_option("c
 # data.frames rbinded together:
 #   * the expanded grid for all factors *not* in nuis, with the nuis factors set 
 #     at their first levels
-#   * for each factor in f nuis, a set of rows for (levs$f), with the other 
+#   * for each factor f in nuis, a set of rows for (levs$f), with the other 
 #     factors at their first levels (this is arbitrary)
 # In addition, we return a character vector 'row.assign' corresponding to the rows 
 # in the grid, telling which is what: ".main.grid." for the first part of the grid,
 # otherwise factor names from nuis.
 # We also return 'nuis' itself - which may be reduced since illegal 
 # entries are silently removed
+###
+# Update July 2024...
+# All this trickery assumes that columns of X are associated with model terms!
+# So this does not work for anything that's been re-gridded.
+# Those (almost?) always have X = I. Unfortunately we won't see X until
+# later, but we can still stop it before we inflict damage...
+# Note: we also provide a simpler check: misc$regrid.flag is non-NULL
 .setup.nuis = function(nuis, levs, trms, rg.limit) {
     firsts =  args = lapply(levs, function(x) x[1])
     nuis = intersect(nuis, names(levs))
@@ -1091,9 +1106,12 @@ ref_grid <- function(object, at, cov.reduce = mean, cov.keep = get_emm_option("c
 # bottom part, and substitute those averages in the required columns in the top part 
 # of the model matrix.
 .basis.nuis = function(basis, info, wt, levs, data, grid, ref.levels) {
+    X = basis$X
+    if(!is.null(basis$misc$regrid.flag) || all(apply(X, 1, \(x) sum(x != 0)) == 1))   # each row has 1 nonzero element
+        stop("Sorry, 'nuisance' specs are not allowed for this situation.",
+             " Revise the call accordingly.", call. = FALSE)
     ra = info$row.assign
     r. = rep(".", length(ra))  # fillers
-    X = basis$X
     n = sum(ra == ".main.grid.")
     k = nrow(X) / length(ra)   # multivariate dimension
     nuis = info$nuis
@@ -1126,22 +1144,138 @@ ref_grid <- function(object, at, cov.reduce = mean, cov.keep = get_emm_option("c
     basis$X = X[RA == ".main.grid.", , drop = FALSE]
     non.nuis = setdiff(names(ref.levels), info$nuis)
     basis$ref.levels = ref.levels[non.nuis]
-    basis$grid = grid[1:n, non.nuis, drop = FALSE]
+    basis$grid = grid[ra == ".main.grid.", non.nuis, drop = FALSE]
     basis
 }
 
+# Internal function to do reference grid for counterfactuals
+.cf.refgrid = function(object, counterfactuals, data, options = list(), ...) {
+    if(missing(data))
+        data = recover_data(object, ...)
+    if(!hasName(data, "(weights)"))
+        pwts = rep(1, nrow(data))
+    else
+        pwts = data[["(weights)"]]
+    
+    # Start with just the ordinary reference grid
+    rg = ref_grid(object, data = data, ...)
+    cfac = intersect(counterfactuals, names(rg@levels))
+    clevs = rg@levels[cfac]
+    cgrid = do.call(expand.grid, clevs)
+    # handle nasty fact that character predictors don't act like factors
+    for (j in cfac)
+        if(is.character(data[[j]])) 
+            cgrid[[j]] = as.character(cgrid[[j]])
+    
+    # Get the stuff we need for each main dataset step
+    link = .get.link(rg@misc)
+    if(is.null(link))
+        link = make.link("identity")
+    trms = rg@model.info$terms
+    xlev = rg@model.info$xlev
+    misc = list()
+    offset = .get.offset(object, grid = data)
+    k = ifelse(length(mr <- rg@roles$multresp) == 0, 1, length(rg@levels[[mr]])) # grid expansion factor
+    
+    # Index sets for combinations of factors
+    pg = apply(cgrid, 1, paste, collapse = ".")
+    pd = apply(data[cfac], 1, paste, collapse = ".")
+    cidx = lapply(pg, \(x) which(pd == x))
 
+    # special case for covariates with no matches
+    if(all(sapply(cidx, length) == 0))
+        cidx = list(seq_len(nrow(data)))
+    
+    # account for any NAs in bhat
+    nonNA = !is.na(rg@bhat)
+    # ensure we include all levels of cfacs with data
+    all.active = sort(unlist(cidx))
+    deadrows = sapply(cidx, function(x) x[1])
+    offset = c(offset, rep(mean(offset), length(deadrows)))
+    pwts = c(pwts, rep(mean(pwts), length(deadrows)))
+    data = rbind(data, data[deadrows, ])
+    n = nrow(data)
+    mymean = function(x) ifelse(is.null(x), NA, mean(x))
+    
+    # get means of groups of prior weights
+    mpwt = sapply(cidx, \(i) mean(pwts[i]))
+    
+    ## Compile the averaged results for delta method
+    DL = matrix(nrow = 0, ncol = sum(nonNA))
+    bh = numeric(0)
+    embmeth = .find_method(object, "emm_basis")
+    for (i in seq_len(nrow(cgrid))) {
+        g = data
+        for(j in cfac)
+            g[all.active, j] = cgrid[i,j]
+        bas = embmeth(object, trms, xlev, g, ...)
+        if(!is.null(bas$misc$postGridHook))
+            stop("Sorry, we do not support counterfactuals for this situation.")
+        X = bas$X[, nonNA, drop = FALSE]
+        eta = X %*% bas$bhat[nonNA]
+        yhat = link$linkinv(eta + offset)
+        d = link$mu.eta(eta) * rep(pwts, k)   # includes prior weights
+        X = sweep(X, 1, d, "*")
+        
+        pos = 0
+        XX = matrix(nrow = 0, ncol = ncol(X))
+        for(I in 1:k) {
+            XX = sapply(cidx, \(i) apply(X[pos + i, , drop = FALSE], 2, mymean)) / mpwt
+            DL = rbind(DL, t(XX))
+            yy = sapply(cidx, \(i) ifelse(length(i) == 0, NA, mean(yhat[i + pos])))
+            bh = c(bh, yy)
+            pos = pos + n
+        }
+    }
+    RG = rg
+    RG@bhat = bh
+    nonNA = !is.na(bh)
+    RG@linfct = diag(nrow(DL))
+    RG@V = DL %*% rg@V %*% t(DL)
+    levs = rg@levels
+    levs[cfac] = NULL
+    alevs = clevs
+    names(alevs) = paste0("actual_", cfac)
+    levs = c(alevs, clevs)
+    if (k > 1)
+        levs = c(levs, rg@levels[length(rg@levels)])
+    RG@levels = levs
+    wts = sapply(cidx, length) * mpwt
+    RG@grid = do.call("expand.grid", levs)
+    RG@grid$.wgt. = rep(wts, length(bh)/length(wts))
+    misc = rg@misc
+    if(!is.null(misc$inv.lbl))
+        misc$estName = misc$inv.lbl
+    misc[c("tran", "inv.lbl", "sigma")] = NULL
+    RG@misc = c(misc, famSize = length(bh), cf.grid = TRUE)
+    RG@model.info$model.matrix = NULL
+    RG@roles$predictors = c(names(alevs), names(clevs))
+    RG@roles$counterfactuals = cfac
+    if (all(nonNA))
+        RG@nbasis = estimability::all.estble
+    else {
+        RG@nbasis = matrix(0, ncol = sum(!nonNA), nrow = length(bh))
+        idx = which(!nonNA)
+        for (j in seq_len(ncol(RG@nbasis)))
+            RG@nbasis[idx[j], j] = 1
+        RG@V = RG@V[nonNA, nonNA]
+    }
+    RG
+}
+
+
+## OLD CODE...
 # Set up grid for counterfactuals - i.e., copies of whole dataset with
 # counterfactual levels substituted, with obs index varying the slowest
-.setup.cf = function(levs, data) {
-    lv = arg = levs[-length(levs)] # omit .obs.no.
-    arg$stringsAsFactors = FALSE
-    g = do.call(expand.grid, arg)
-    idx = rep(seq_len(nrow(data)), each = nrow(g))
-    xdata = data[idx, , drop = FALSE]
-    idx = matrix(seq_len(nrow(xdata)), nrow = nrow(g))
-    for (i in seq_along(g[, 1]))
-        for (j in names(lv))
-            xdata[idx[i, ], j] = g[i, j]
-    xdata
-}
+# .setup.cf = function(levs, data) {
+#     lv = arg = levs[-length(levs)] # omit .obs.no.
+#     arg$stringsAsFactors = FALSE
+#     g = do.call(expand.grid, arg)
+#     idx = rep(seq_len(nrow(data)), each = nrow(g))
+#     xdata = data[idx, , drop = FALSE]
+#     idx = matrix(seq_len(nrow(xdata)), nrow = nrow(g))
+#     for (i in seq_along(g[, 1]))
+#         for (j in names(lv))
+#             xdata[idx[i, ], j] = g[i, j]
+#     xdata
+# }
